MODULE sed
   !!======================================================================
   !!                        ***  sed  ***
   !! Sediment :   set sediment global variables
   !!======================================================================
#if defined key_sed
   !! History :
   !!        !  06-12  (C. Ethe)  Orignal
   !!----------------------------------------------------------------------
   USE par_sed

   USE in_out_manager

   USE dom_oce , ONLY :       &
      nidom    =>   nidom  ,  & !:
      glamt    =>   glamt  ,  & !: longitude of t-point (degre)
      gphit    =>   gphit  ,  & !: latitude  of t-point (degre)
      e3t_0    =>   e3t_0  ,  & !: reference depth of t-points (m)
      mbathy   =>   mbathy ,  & !: bathymetry
      tmask    =>   tmask  ,  & !: land/ocean mask at t-points
      rdt      =>   rdt         !: time step for the dynamics

      nyear     =>   nyear    ,   & !: Current year
      nmonth    =>   nmonth   ,   & !: Current month
      nday      =>   nday     ,   & !: Current day
      ndastp    =>   ndastp   ,   & !: time step date in year/month/day aammjj
      nday_year =>   nday_year,   & !: curent day counted from jan 1st of the current year
      adatrj    =>   adatrj         !: number of elapsed days since the begining of the run
   !                                !: it is the accumulated duration of previous runs
   !                                !: that may have been run with different time steps.


#if ! defined key_sed_off

   USE oce , ONLY :            &
      tsn      =>    tsn       & !: pot. temperature (celsius) and salinity (psu)

   USE trc, ONLY :  &
      trn        , & !: tracer 
      nwritetrc      !: outputs frequency of tracer model

   USE p4zsink, ONLY :  &
      sinking  ,     & !: sinking flux for POC
#if ! defined key_kriest
      sinking2 ,     & !: sinking flux for GOC
#endif
      sinkcal ,      & !: sinking flux for calcite 
      sinksil          !: sinking flux for opal ( dsi )

   USE sms_pisces, ONLY :  &
      akb3  ,     & 
      ak13  ,     & 
      ak23  ,     & 
      akw3  ,     & 
      aksp  ,     & 
      borat 

#endif   

   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpdta) :: &
      trc_data           !: tracer data to share with sediment model

   INTEGER, PUBLIC :: &
      nitsed000, nitsedend, nwrised, nfreq

   REAL(wp), PUBLIC, DIMENSION(:,:,:) , ALLOCATABLE ::   &
      pwcp  ,      &     !: pore water sediment data at given time-step
      pwcp0              !: pore water sediment data at initial time

   REAL(wp), PUBLIC, DIMENSION(:,:,:) , ALLOCATABLE ::   &
      solcp ,      &     !: solid sediment data at given time-step
      solcp0             !: solid sediment at initial time

   !! * Shared module variables
   REAL(wp), PUBLIC, DIMENSION(:,:) , ALLOCATABLE ::   &
      pwcp_dta           !: pore water data at given time-step

   REAL(wp), PUBLIC, DIMENSION(:,:) , ALLOCATABLE ::   &
      rainrm_dta,  &     !: rain data at at initial time
      rainrm    ,  &     !: rain data at given time-step
      rainrg    ,  &     !: rain of each solid component in [g/(cm**2.s)]
      fromsed   ,  &     !:
      tosed     ,  &     !:
      rloss              !: 

   REAL(wp), PUBLIC, DIMENSION(:,:) , ALLOCATABLE  ::   &
      tokbot             !: 

   REAL(wp), PUBLIC, DIMENSION(:) , ALLOCATABLE  ::   & 
      temp  ,      &     !: temperature
      salt  ,      &     !: salinity
      press ,      &     !: pressure
      raintg,      &     !: total massic flux rained in each cell (sum of sol. comp.)
      dzdep              !: total thickness of solid material rained [cm] in each cell

   REAL(wp), PUBLIC, DIMENSION(:,:) , ALLOCATABLE  ::   &
      hipor  ,     &     !: [h+] in mol/kg*densSW 
      co3por             !: [co3--]solid sediment at initial time

   REAL(wp), PUBLIC, DIMENSION(jpsol) ::   &
      mol_wgt            !: molecular weight of solid sediment data

   
   !! Geometry
   INTEGER, PUBLIC, SAVE :: &
      jpoce ,  &             !: sediment points
      indoce

   INTEGER , PUBLIC, DIMENSION(:) , ALLOCATABLE  ::   &  !:
      iarroce      !: ! Computation of 1D array of sediments points

   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) :: &
      epkbot             !: ocean bottom layer thickness

   REAL(wp), PUBLIC, DIMENSION(:) , ALLOCATABLE  :: &
      dzkbot             !: ocean bottom layer thickness in meters

   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpksed) :: &
      tmasksed           !: sediment mask

   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) :: &
      sbathy
 
   REAL(wp), PUBLIC, DIMENSION(jpksed) :: &
      dz      ,    &     !: sediment layers thickness
      por     ,    &     !: porosity profile     
      por1    ,    &     !: 1 - por 
      profsed ,    &     !: depth of middle of each layer
      volw    ,    &     !: volume of pore water cell fraction
      vols    ,    &     !: volume of solid cell fraction
      diff    ,    &     !: diffusion ceofficient
      rdtsed             !: 

   REAL(wp), PUBLIC, DIMENSION(:,:) , ALLOCATABLE :: &
      dz3d   ,     &     !: 
      volw3d ,     &     !: 
      vols3d             !: 

   REAL(wp)  ::    &
      dens               !: density of solid material

   !! Namelist
   REAL(wp), PUBLIC, DIMENSION(5) :: &   
      reac               !: reactivity rc in  [l.mol-1.s-1]

   REAL(wp), PUBLIC       :: &   
      sat_sil,      &     !: saturation concentration for silicate in [mol.l-1]
      sat_clay,     &     !:                              clay    
      reac_sil,     &     !: reactivity rc in  [l.mol-1.s-1]
      reac_clay,    &     
      reac_poc,     &     
      reac_no3,     &     
      reac_cal         

   REAL(wp), PUBLIC    :: &   
      so2ut   ,     &     !:
      srno3   ,     &     !: 
      spo4r   ,     &     !:
      srDnit  ,     &     !: 
      sthro2              !: threshold O2 concen. in [mol.l-1]

   REAL(wp), PUBLIC    :: &   
      pdb    = 0.0112372, & !: 13C/12C in PD Belemnite
      rc13P  = 0.980    , & !: 13C/12C in POC = rc13P*PDB
      rc13Ca = 1.001        !: 13C/12C in CaCO3 = rc13Ca*PDB

   REAL(wp) , PUBLIC ::   &
      dtsed  ,     &     !: sedimentation time step
      db                 !: bioturb coefficient in [cm2.s-1]

   !! Chemistry
   REAL(wp), PUBLIC, DIMENSION(:) , ALLOCATABLE  :: &
      densSW ,     &
      borats ,     &
      calcon2,     &
      akbs   ,     &
      ak1s   ,     &
      ak2s   ,     &
      akws   ,     &
      ak12s  ,     &
      ak1ps  ,     &
      ak2ps  ,     &
      ak3ps  ,     &
      ak12ps ,     &
      ak123ps,     &
      aksis  ,     & 
      aksps 

   !! Inputs / Outputs
   CHARACTER( len = 80 ), DIMENSION(jptrased) :: &
      sedtrcl
   CHARACTER( len = 20 ), DIMENSION(jptrased) :: &
      sedtrcd , sedtrcu

   CHARACTER( len = 80 ), DIMENSION(jpdia3dsed) :: &
      seddia3l 
   CHARACTER( len = 20 ), DIMENSION(jpdia3dsed) :: &
      seddia3d , seddia3u

   CHARACTER( len = 80 ), DIMENSION(jpdia2dsed) :: &
      seddia2l 
   CHARACTER( len = 20 ), DIMENSION(jpdia2dsed) :: &
      seddia2d , seddia2u

   REAL(wp), DIMENSION(jpi,jpj,jpksed,jptrased) :: &
      trcsedi
   REAL(wp), DIMENSION(jpi,jpj,jpksed,jpdia3dsed) :: &
      flxsedi3d
   REAL(wp), DIMENSION(jpi,jpj,jpdia2dsed) :: &
      flxsedi2d

   !! Units
   INTEGER, PUBLIC :: &
     numsed = 27
#else
   !!======================================================================
   !! No Sediment model
   !!======================================================================
#endif

END MODULE sed
